
# q-expansion of newform 2160.2.q.f, downloaded from the LMFDB on 28 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2160
weight = 2
poly_data = [9, -3, -2, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[-3, -2, 2, 1], 6], [[0, 5, 1, -1], 3], [[-9, 2, 1, 2], 3]]

hecke_ring_character_values = [[271, [1, 0, 0, 0]], [1621, [1, 0, 0, 0]], [2081, [-1, 1, 0, 0]], [1297, [1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 0, 0, 0], [-1, 1, 0, 0], [0, 0, 0, 1], [0, 1, 0, 1], [2, 0, 2, -2], [4, 0, -1, 0], [0, 0, 1, 0], [1, -2, -1, 1], [0, -2, 0, 1], [0, 2, 2, -2], [-4, 0, 0, 0], [-3, 3, 0, 0], [0, -9, 0, 1], [0, 4, 0, 1], [2, 0, 4, 0], [-2, 1, -1, 1], [0, -2, 0, 3], [-7, 7, 0, 0], [-6, 0, 0, 0], [-4, 0, 3, 0], [0, 2, 0, 0], [0, -4, 0, -1], [-9, 0, -3, 0], [0, 5, 0, 1], [0, -4, 0, 2], [-16, 16, 0, 0], [-1, 0, 4, 0], [13, 0, 1, 0], [8, -10, -2, 2], [1, 0, 3, 0], [-2, 4, 2, -2], [0, -9, 0, -3], [8, -11, -3, 3], [9, -12, -3, 3], [0, -10, 0, 0], [2, 0, 2, -2], [12, 0, 4, 0], [-11, 10, -1, 1], [0, 14, 0, 2], [-10, 0, -2, 0], [9, 0, 5, 0], [0, -4, 0, -4], [-14, 11, -3, 3], [-4, 0, 4, 0], [-4, 0, -4, 0], [-14, 12, -2, 2], [0, 10, 0, -1], [0, -9, 0, -3], [-15, 16, 1, -1], [0, 0, -3, 0], [8, -10, -2, 2], [0, -1, 0, 4], [-8, 0, -7, 0], [2, -1, 1, -1], [0, 4, 0, 4], [-1, 0, 1, 0], [-8, 0, 0, 0], [0, -10, 0, -2], [0, 2, 0, -1], [-15, 20, 5, -5], [12, -6, 6, -6], [13, 0, -6, 0], [16, -14, 2, -2], [0, -7, 0, 1], [0, -4, 0, 2], [0, -4, 0, 6], [0, 1, 1, -1], [12, -15, -3, 3], [0, -8, 0, -3], [0, 9, 0, -9], [-8, 0, -4, 0], [0, -16, 0, 0], [6, -2, 4, -4], [-6, 0, -5, 0], [4, 4, 8, -8], [0, -8, 0, 1], [18, 0, 2, 0], [6, -9, -3, 3], [-6, 1, -5, 5], [-2, -8, -10, 10], [0, 10, 0, -12], [-6, 0, 6, 0], [4, 0, 1, 0], [0, 2, 0, -6], [0, 17, 0, -4], [2, 0, -5, 0], [0, -23, 0, 3], [0, 12, 0, -3], [20, -20, 0, 0], [36, 0, 3, 0], [0, 14, 0, -4], [8, 0, 2, 0], [20, -13, 7, -7], [8, -5, 3, -3], [5, 0, -5, 0], [-21, 12, -9, 9], [21, 0, -6, 0], [1, 0, -3, 0], [25, 0, 1, 0], [0, 11, 0, -12], [30, 0, 6, 0], [17, -7, 10, -10], [0, -7, 0, -7], [-8, 15, 7, -7], [-34, 0, -3, 0], [0, -27, 0, 0], [-4, 0, 10, 0], [18, -24, -6, 6], [0, -15, 0, 5], [11, -8, 3, -3], [20, 0, 6, 0], [-12, 15, 3, -3], [0, -31, 0, 3], [0, 0, 10, 0], [0, -39, 0, 0], [11, -11, 0, 0], [-13, 0, -11, 0], [-4, 14, 10, -10], [0, -4, 0, -4], [-2, 8, 6, -6], [0, 10, 0, 0], [0, -20, 0, 10], [-4, 0, -11, 0], [0, -16, 0, -6], [-31, 0, -5, 0], [0, 14, 0, -5], [-18, 0, 6, 0], [0, 8, 0, 3], [-14, 20, 6, -6], [34, 0, 3, 0], [-15, 18, 3, -3], [2, 4, 6, -6], [-10, 0, 0, 0], [-25, 14, -11, 11], [35, -30, 5, -5], [-8, 0, 2, 0], [-28, 28, 0, 0], [10, -8, 2, -2], [-18, 0, -9, 0], [-6, 0, 1, 0], [0, 20, 0, -13], [1, -12, -11, 11], [25, 0, 5, 0], [-43, 0, -3, 0], [0, -30, 0, 6], [0, 16, 0, 6], [0, 14, 0, -4], [-10, 7, -3, 3], [-19, 0, 7, 0], [2, -18, -16, 16], [3, 0, 9, 0], [21, 0, 10, 0], [4, -14, -10, 10], [0, -7, 0, 0], [0, 42, 0, 0], [-2, 0, 12, 0], [0, -2, 0, 16], [16, 0, 10, 0], [41, -40, 1, -1], [0, 1, 0, 10], [-14, 0, 5, 0], [29, -26, 3, -3], [-4, 0, -8, 0], [-30, 33, 3, -3], [0, -4, 0, 5], [-2, 0, -6, 0], [0, -14, 0, 0], [5, 0, 15, 0], [8, -10, -2, 2], [0, 17, 0, 11], [-41, 44, 3, -3], [26, -34, -8, 8], [0, -9, 0, 5], [28, -18, 10, -10], [2, 2, 4, -4], [0, -11, 0, -1], [58, 0, 2, 0], [-43, 0, 5, 0], [0, -24, 0, -1], [0, 4, 0, -4], [0, -1, 0, -19], [-8, -4, -12, 12], [46, 0, -1, 0], [7, -8, -1, 1], [0, -14, 0, 13], [-4, 0, -6, 0], [0, 40, 0, -7], [-15, 25, 10, -10], [34, 0, -4, 0], [-42, 0, 5, 0], [0, 23, 0, 2], [-16, 0, 5, 0], [0, -24, 0, -9], [-51, 0, 0, 0], [-26, 19, -7, 7], [-29, 23, -6, 6], [0, -14, 0, -6], [0, -47, 0, -5], [-5, 0, 11, 0], [2, -16, -14, 14], [0, 44, 0, -6], [16, -4, 12, -12], [0, 49, 0, -6], [-24, 0, -3, 0], [-14, 0, 2, 0], [-54, 0, -2, 0], [23, -19, 4, -4], [0, 39, 0, -6], [12, -19, -7, 7], [-6, 0, 11, 0], [-26, 22, -4, 4], [0, -22, 0, 3], [0, -36, 0, 0], [14, -22, -8, 8], [0, -17, 0, 0], [13, -18, -5, 5], [0, 25, 0, 13], [-1, 0, 13, 0], [12, -18, -6, 6], [57, -50, 7, -7], [2, -14, -12, 12], [13, 7, 20, -20], [-11, 0, 3, 0], [15, 3, 18, -18], [0, 24, 0, 14], [0, -54, 0, 6], [30, 0, 12, 0], [0, 10, 0, 17], [0, -4, 0, -16], [14, -18, -4, 4], [10, 0, -6, 0], [34, -30, 4, -4], [17, -19, -2, 2], [0, -4, 0, -9], [0, -22, 0, 5], [1, 2, 3, -3], [30, 0, 6, 0], [12, -15, -3, 3], [2, 0, -14, 0], [0, 37, 0, 10], [34, 0, 6, 0], [-19, 22, 3, -3], [35, 0, -3, 0], [4, -17, -13, 13], [0, 36, 0, -12], [3, 0, -5, 0], [-52, 47, -5, 5], [0, -4, -4, 4], [7, 0, -19, 0], [4, -22, -18, 18], [-19, 0, 16, 0], [-16, 2, -14, 14], [0, -38, 0, 3], [0, 28, 0, 4], [-52, 0, 1, 0], [-34, 0, -12, 0], [0, -52, 0, 9], [-32, 0, -16, 0], [56, 0, -3, 0], [0, 22, 0, -13], [0, -69, 0, 0], [-26, 14, -12, 12], [-38, 0, -8, 0], [-10, 26, 16, -16], [0, 11, 0, -9], [-26, 0, 8, 0], [0, -9, 0, 0], [-13, 7, -6, 6], [-6, 0, -6, 6], [14, -18, -4, 4], [-23, 0, -3, 0], [0, 43, 0, -15], [-10, -2, -12, 12], [12, -23, -11, 11], [-5, 0, -15, 0], [-77, 76, -1, 1], [0, -16, 0, -2], [-27, 0, 2, 0], [0, -59, 0, 1], [16, -26, -10, 10], [2, -14, -12, 12], [0, -8, 0, -8], [0, 10, 0, -15], [11, -23, -12, 12], [-76, 0, -2, 0], [-14, 0, 4, 0], [16, -20, -4, 4], [0, 56, 0, -12], [-7, 0, 4, 0], [0, 6, 0, -12], [44, 0, -11, 0], [16, 7, 23, -23], [-20, 25, 5, -5], [0, -14, 0, -18], [41, -46, -5, 5], [0, -26, 0, -16], [0, -54, 0, -6], [30, 0, 18, 0], [0, -76, 0, 0], [-8, 23, 15, -15], [-16, 0, -14, 0], [-4, 0, 20, 0]]
